//
//  FMODaudio.m
//  bangbang
//
//  Created by Peter Drescher on 6/15/12.
//  Copyright (c) 2012 Twittering Machine. All rights reserved.
//

#import "FMODaudio.h"
#include "bangbang.h"

@implementation FMODaudio

void ERRCHECK(FMOD_RESULT result)
{
    if (result != FMOD_OK)
    {
        fprintf(stderr, "FMOD error! (%d) %s\n", result, FMOD_ErrorString(result));
        exit(-1);
    }
}

- (void)FMODviewDidLoad
{
    eventSystem = NULL;
    bangEvent   = NULL;
    sparkEvent  = NULL;
    fireEvent  = NULL;
}

- (void)FMODviewWillAppear
{
    FMOD_RESULT result      = FMOD_OK;
    char        buffer[200] = {0};
    FMOD_REVERB_PROPERTIES fireRvb;
    
    result = FMOD::EventSystem_Create(&eventSystem); 
    ERRCHECK(result);
    
    result = eventSystem->init(32, FMOD_INIT_NORMAL | FMOD_INIT_ENABLE_PROFILE, NULL, FMOD_EVENT_INIT_NORMAL);
    ERRCHECK(result);
    
    [[NSString stringWithFormat:@"%@/bangbang.fev", [[NSBundle mainBundle] resourcePath]] getCString:buffer maxLength:200 encoding:NSASCIIStringEncoding];
    result = eventSystem->load(buffer, NULL, NULL);
    ERRCHECK(result);
    
    result = eventSystem->getReverbPresetByIndex(EVENTREVERB_BANGBANG_FIRERVB, &fireRvb);
    ERRCHECK(result);
    result = eventSystem->setReverbProperties(&fireRvb);
    ERRCHECK(result);
    
   
    // initialize events to prevent pause during gameplay 
    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_SIMPLEBANGS_BANGS_4TH, FMOD_EVENT_DEFAULT, &bangEvent);  ERRCHECK(result);
    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_CANNON_SPARKS,         FMOD_EVENT_DEFAULT, &sparkEvent); ERRCHECK(result);
    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_CANNON_LAZER,          FMOD_EVENT_DEFAULT, &fireEvent);  ERRCHECK(result);
    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_CANNON_AIMING,         FMOD_EVENT_DEFAULT, &miscEvent);  ERRCHECK(result);

    updateTimer = [NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(timerUpdate:) userInfo:nil repeats:YES];
}


- (void)FMODviewWillDisappear
{
    [updateTimer invalidate];
    
    if (eventSystem)
    {
        eventSystem->release();
        eventSystem = NULL;
    }    
}


- (void)timerUpdate:(NSTimer *)timer 
{
    FMOD_RESULT         result  = FMOD_OK;
    
    result = eventSystem->update();
    ERRCHECK(result);
}

- (void)playBang: (int) whichBang
{
    FMOD_RESULT result = FMOD_OK;
    switch (whichBang) {
        case 0:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_SIMPLEBANGS_BANGS_4TH,     FMOD_EVENT_DEFAULT, &bangEvent);
            break;
        case 1:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_SIMPLEBANGS_BANGS_BALLON,  FMOD_EVENT_DEFAULT, &bangEvent);
            break;
        case 2:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_SIMPLEBANGS_BANGS_NI3,     FMOD_EVENT_DEFAULT, &bangEvent);
            break;
        case 3:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_SIMPLEBANGS_BANGS_GARAGE,  FMOD_EVENT_DEFAULT, &bangEvent);
            break;
        case 4:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_SIMPLEBANGS_BANGS_WS,      FMOD_EVENT_DEFAULT, &bangEvent);
            break;
        case 5:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_COMPLEXBANGS_COMPLEXBANG1, FMOD_EVENT_DEFAULT, &bangEvent);
            break;
        case 6:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_COMPLEXBANGS_COMPLEXBANG2, FMOD_EVENT_DEFAULT, &bangEvent);
            break;
        case 7:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_COMPLEXBANGS_COMPLEXBANG3, FMOD_EVENT_DEFAULT, &bangEvent);
            break;
        case 8:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_FXBANGS_FXBANG_FLANGE,     FMOD_EVENT_DEFAULT, &bangEvent);
            break;
        case 9:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_FXBANGS_FXBANG_ECHO,       FMOD_EVENT_DEFAULT, &bangEvent);
            break;            
        case 10:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_BONUS_GLASS,               FMOD_EVENT_DEFAULT, &bangEvent);
            break;            
        case 11:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_BONUS_BATCHORDS,           FMOD_EVENT_DEFAULT, &bangEvent);
            break;            
        case 12:
            result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_BONUS_BONG,                FMOD_EVENT_DEFAULT, &bangEvent);
            break;            
        default:
            NSLog(@"playBang bad mojo - whichBang = %d", whichBang);
            break;
    }
    ERRCHECK(result);
    
    result = bangEvent->start();
    ERRCHECK(result);
    //[self stopSparks];
}

    
- (void)playAiming:(float)panAmt
{
    FMOD_RESULT result = FMOD_OK;
    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_CANNON_AIMING, FMOD_EVENT_DEFAULT, &miscEvent);
    ERRCHECK(result);
    result = miscEvent->getParameterByIndex(EVENTPARAMETER_BANGBANG_CANNON_AIMING_PAN, &eventParameter);
    ERRCHECK(result);
    result = eventParameter->setValue(panAmt);
    ERRCHECK(result);
    result = miscEvent->start();
    ERRCHECK(result);
}

- (void)playCannonFire
{
    FMOD_RESULT result = FMOD_OK;
    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_CANNON_ZAP, FMOD_EVENT_DEFAULT, &fireEvent);
    ERRCHECK(result);
    result = fireEvent->start();
    ERRCHECK(result);
}

- (void)playMGFire
{
    FMOD_RESULT result = FMOD_OK;
    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_CANNON_MGFIRE, FMOD_EVENT_DEFAULT, &fireEvent);
    ERRCHECK(result);
    result = fireEvent->start();
    ERRCHECK(result);
}

- (void)playMortar
{
    FMOD_RESULT result = FMOD_OK;
    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_CANNON_MORTAR, FMOD_EVENT_DEFAULT, &fireEvent);
    ERRCHECK(result);
    result = fireEvent->start();
    ERRCHECK(result);

    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_CANNON_SPARKS, FMOD_EVENT_DEFAULT, &sparkEvent);
    ERRCHECK(result);
    result = sparkEvent->start();
    ERRCHECK(result);
}

- (void)setSparkVol:(float)sparkVol
{
    FMOD_RESULT result = FMOD_OK;
    if (sparkVol > 1.0) {
        [self stopSparks];
    } else {
        result = sparkEvent->getParameterByIndex(EVENTPARAMETER_BANGBANG_CANNON_SPARKS_VOL, &eventParameter);
        ERRCHECK(result);
        result = eventParameter->setValue(sparkVol);
        ERRCHECK(result);
    }
}

- (void)stopSparks
{
    FMOD_RESULT result = FMOD_OK;
    result = sparkEvent->stop();
    ERRCHECK(result);
}

- (void)playLazer
{
    FMOD_RESULT result = FMOD_OK;
    result = eventSystem->getEventBySystemID(EVENTID_BANGBANG_CANNON_LAZER, FMOD_EVENT_DEFAULT, &fireEvent);
    ERRCHECK(result);
    result = fireEvent->start();
    ERRCHECK(result);
}

- (void)stopLazer
{
    FMOD_RESULT result = FMOD_OK;
    result = fireEvent->stop();
    ERRCHECK(result);
}

- (void)dealloc
{   
	[super dealloc];
}

@end
