//
//  LevelView.m
//  bangbang
//
//  Created by Peter Drescher on 6/10/12.
//  Copyright (c) 2012 Twittering Machine. All rights reserved.
//

#import "LevelView.h"

#define kFireballSize    15
#define kShotSpeed       20
#define kNumberOfTargets 4
#define kRedTarget       0
#define kGreenTarget     1
#define kBlueTarget      2
#define kBonusTarget     3
#define kBonusGlass      0
#define kBonusBatSignal  1
#define kBonusTriangle   2
#define kFieldEdge       320
#define kFireModeCannon     0
#define kFireModeMachineGun 1
#define kFireModeMortar     2
#define kFireModeLazer      3

@implementation LevelView

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    viewRect = frame;
    if (self) {
        [self initLevel];
    }
    return self;
}

- (void) initLevel
{
    bgImage = [UIImage imageNamed:@"320x415bg6a.png"];
    bgImageRect = CGRectMake(0, 0, 320, 395);
        
    //initialize and store target properties
    targetArray = [[NSMutableArray alloc] init ];
    for (int t=0; t<kNumberOfTargets; t++) {
        Targets *tgt = [Targets new];
        switch (t) {
            case 0:
                tgt.targetRect = CGRectMake(0, 50, 20, 20);
                tgt.targetColor = [UIColor redColor].CGColor;
                tgt.targetSpeed = 3;
                tgt.targetSlopeAmt = (arc4random() % 10)+1;
                tgt.nextPos = CGPointMake(kFieldEdge, kFieldEdge);
                tgt.fadeCtr = 1.0;
                break;
            case 1:
                tgt.targetRect = CGRectMake(0, 100, 40, 40);
                tgt.targetColor = [UIColor greenColor].CGColor;
                tgt.targetSpeed = 5;
                tgt.targetSlopeAmt = (arc4random() % 10)+1;
                tgt.nextPos = CGPointMake(kFieldEdge, kFieldEdge);
                tgt.fadeCtr = 1.0;
                break;
            case 2:
                tgt.targetRect = CGRectMake(0, 150, 60, 60);
                tgt.targetColor = [UIColor blueColor].CGColor;
                tgt.targetSpeed = 7;
                tgt.targetSlopeAmt = (arc4random() % 10)+1;
                tgt.nextPos = CGPointMake(kFieldEdge, kFieldEdge);
                tgt.fadeCtr = 1.0;
                break;
            case 3: // bonus
                tgt.targetRect = CGRectMake(-60, 150, 30, 30);
                tgt.targetColor = [UIColor whiteColor].CGColor;
                tgt.targetSpeed = 3;
                tgt.targetSlopeAmt = 0; //ignored
                tgt.nextPos = CGPointMake(kFieldEdge, kFieldEdge);
                tgt.fadeCtr = 1.0;
                //tgt.targetHit = true;
                break;
         }
         [targetArray insertObject:tgt atIndex:t];
    }
    
    //initialize images
    bangImage   = [UIImage imageNamed:@"bang.png"];
    powImage    = [UIImage imageNamed:@"pow.png"];
    bamImage    = [UIImage imageNamed:@"bam.png"];
    whamImage   = [UIImage imageNamed:@"wham.png"];
    blammoImage = [UIImage imageNamed:@"blammo.png"];
    blamImage   = [UIImage imageNamed:@"blam.png"];
    krackImage  = [UIImage imageNamed:@"krack.png"];
    boomImage   = [UIImage imageNamed:@"boom.png"];
    zotImage    = [UIImage imageNamed:@"zot.png"];
    kaboomImage = [UIImage imageNamed:@"kaboom.png"];
    batSignal   = [UIImage imageNamed:@"batman.png"];
    glassImage  = [UIImage imageNamed:@"glass.png"];
    batImage[0] = [UIImage imageNamed:@"whap.png"];
    batImage[1] = [UIImage imageNamed:@"biff.png"];
    batImage[2] = [UIImage imageNamed:@"oooof.png"];
    batImageCtr = 0;
    
    shotFired = false;
    hitCounter = 0;
    fireMode = kFireModeCannon;
    //fireMode = kFireModeMachineGun;
    //fireMode = kFireModeLazer;
    //fireMode = kFireModeMortar;
    lazerOn = false;
    lazerClrCtr = 0;
    mgFireCtr = 0;
    mgBulletCtr = 0;
    for (int i=0; i<5; i++) {
        mgBullets[i] = CGRectMake(0,0,kFireballSize/2, kFireballSize/2);
    }
    bonusTargetType = 0;
    triAngleDx = 0;
    showBonusTarget = false;
    [NSTimer scheduledTimerWithTimeInterval:20.0 target:self selector:@selector(bonusTargetOn) userInfo:nil repeats:NO];
}

- (void)drawRect:(CGRect)rect
{        
    CGContextRef context = UIGraphicsGetCurrentContext();
    
    [self drawBackground:   context];
    
    [self drawTargets:      context];

    [self drawFireball:     context];
    
    [self drawExplosions:   context];

}

- (void) drawBackground: (CGContextRef) context {
    
    // draw background image
    CGContextDrawImage(context, bgImageRect, bgImage.CGImage);
    
    //draw inner square
    CGContextSetRGBStrokeColor(context,1.0,1.0,1.0,0.3);
    CGContextMoveToPoint(context, 52, kFieldEdge - 33);
    CGContextAddLineToPoint(context, kFieldEdge - 52, kFieldEdge - 33);
    CGContextAddLineToPoint(context, kFieldEdge - 52, 56);
    CGContextAddLineToPoint(context, 52, 56);
    CGContextAddLineToPoint(context, 52, kFieldEdge - 33);
    CGContextStrokePath(context);
    
    // draw aiming line
    switch (fireMode) {
        case kFireModeCannon:
            CGContextSetLineWidth(context, 10.0);
            CGContextSetStrokeColorWithColor(context, [UIColor blueColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
            CGContextSetLineWidth(context, 5.0);
            CGContextSetStrokeColorWithColor(context, [UIColor purpleColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
            CGContextSetLineWidth(context, 1.0);
            CGContextSetStrokeColorWithColor(context, [UIColor redColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
        break;
    
        case kFireModeMachineGun:
            CGContextSetLineWidth(context, 10.0);
            CGContextSetStrokeColorWithColor(context, [UIColor grayColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
            CGContextSetLineWidth(context, 5.0);
            CGContextSetStrokeColorWithColor(context, [UIColor blueColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
            CGContextSetLineWidth(context, 1.0);
            CGContextSetStrokeColorWithColor(context, [UIColor whiteColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
        break;
    
        case kFireModeMortar:
            CGContextSetLineWidth(context, 10.0);
            CGContextSetStrokeColorWithColor(context, [UIColor darkGrayColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
            CGContextSetLineWidth(context, 5.0);
            CGContextSetStrokeColorWithColor(context, [UIColor lightGrayColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
            CGContextSetLineWidth(context, 1.0);
            CGContextSetStrokeColorWithColor(context, [UIColor yellowColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
        break;
    
        case kFireModeLazer:
            CGContextSetLineWidth(context, 10.0);
            CGContextSetStrokeColorWithColor(context, [UIColor redColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
            CGContextSetLineWidth(context, 5.0);
            CGContextSetStrokeColorWithColor(context, [UIColor orangeColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
            CGContextSetLineWidth(context, 1.0);
            CGContextSetStrokeColorWithColor(context, [UIColor yellowColor].CGColor);
            CGContextMoveToPoint   (context, aimThumbPt.x, aimThumbPt.y);
            CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
            CGContextStrokePath(context);
        break;
    }
}

- (void) drawTargets: (CGContextRef) context {
    // process targets
    CGContextSetLineWidth(context, 2.0);
    for (int t=0; t<kNumberOfTargets; t++) {
        Targets *tgt = [targetArray objectAtIndex:t];
        if (!tgt.targetHit) {
            if (t == kBonusTarget) {
                if (showBonusTarget) {
                    tgt = [self drawBonusTarget:context target:tgt];
                }
            } else {
                CGContextSetStrokeColorWithColor(context, tgt.targetColor);
                
                //bounce off walls
                float newX, newY;
                if (tgt.targetRect.origin.x < tgt.nextPos.x) {
                    newX = tgt.targetRect.origin.x + tgt.targetSpeed;
                    if (newX >= kFieldEdge - tgt.targetRect.size.width) {
                        tgt.nextPos = CGPointMake(0, tgt.nextPos.y);
                    }
                } else {
                    newX = tgt.targetRect.origin.x - tgt.targetSpeed;
                    if (newX <= 0) {
                        tgt.nextPos = CGPointMake(kFieldEdge, tgt.nextPos.y);
                    }
                }
                
                if (tgt.targetRect.origin.y < tgt.nextPos.y) {
                    newY = tgt.targetRect.origin.y + (tgt.targetSpeed/tgt.targetSlopeAmt);
                    if (newY >= kFieldEdge - tgt.targetRect.size.height - 33) {
                        tgt.nextPos = CGPointMake(tgt.nextPos.x, 0);
                    }
                } else {
                    newY = tgt.targetRect.origin.y - (tgt.targetSpeed/tgt.targetSlopeAmt);
                    if (newY <= 0) {
                        tgt.nextPos = CGPointMake(tgt.nextPos.x, kFieldEdge);
                    }
                }
                
                //draw target
                tgt.targetRect = CGRectMake(newX, newY, tgt.targetRect.size.width, tgt.targetRect.size.height);
                CGContextStrokeEllipseInRect(context, tgt.targetRect);
                if (fireMode == kFireModeLazer) { // put a dot in center of target
                    CGContextSetFillColorWithColor(context, tgt.targetColor);
                    CGContextFillRect(context, CGRectMake(tgt.targetRect.origin.x + tgt.targetRect.size.width/2,
                                                          tgt.targetRect.origin.y + tgt.targetRect.size.height/2, 2, 2));
                }
            }
        }
        [targetArray replaceObjectAtIndex:t withObject:tgt];
    }
}

- (Targets*) drawBonusTarget:(CGContextRef)context target:(Targets*)tgt {
             
    //sine waves across screen
    float newX = tgt.targetRect.origin.x + tgt.targetSpeed;
    float radX = (newX / kFieldEdge) * 3*M_PI;
    float newY = tgt.targetRect.origin.y + (3 * sin(radX)); 
    if (newX > kFieldEdge + 50) { //reset
        showBonusTarget = false;
        tgt.targetRect = CGRectMake(-60,  arc4random() % (kFieldEdge/2),tgt.targetRect.size.width, tgt.targetRect.size.height);
        [NSTimer scheduledTimerWithTimeInterval:20.0 target:self selector:@selector(bonusTargetOn) userInfo:nil repeats:NO];  
    } else { //draw bonus target
        tgt.targetRect = CGRectMake(newX, newY, tgt.targetRect.size.width, tgt.targetRect.size.height);
        switch (bonusTargetType) {
            case kBonusGlass: //draw square
                CGContextSetRGBStrokeColor(context,1.0,1.0,1.0,1.0);
                CGContextStrokeRect(context, tgt.targetRect);
                break;
            case kBonusBatSignal: //bat signal
                CGContextSaveGState(context);
                CGContextTranslateCTM(context, 0, batSignal.size.height);
                CGContextScaleCTM(context, 1.0, -1.0);
                CGContextDrawImage(context, CGRectMake(tgt.targetRect.origin.x, -tgt.targetRect.origin.y, batSignal.size.width, batSignal.size.height), batSignal.CGImage);
                CGContextRestoreGState(context);
                break;
            case kBonusTriangle: { //rotating triangle
                CGContextSetFillColorWithColor(context, [UIColor magentaColor].CGColor);
                float v0x = tgt.targetRect.origin.x + ((cos(triAngleDx) * tgt.targetRect.size.width));
                float v0y = tgt.targetRect.origin.y + ((sin(triAngleDx) * tgt.targetRect.size.width));
                float v1x = tgt.targetRect.origin.x + ((cos(2*M_PI/3 + triAngleDx) * tgt.targetRect.size.width));
                float v1y = tgt.targetRect.origin.y + ((sin(2*M_PI/3 + triAngleDx) * tgt.targetRect.size.width));
                float v2x = tgt.targetRect.origin.x + ((cos(4*M_PI/3 + triAngleDx) * tgt.targetRect.size.width));
                float v2y = tgt.targetRect.origin.y + ((sin(4*M_PI/3 + triAngleDx) * tgt.targetRect.size.width));
                CGContextMoveToPoint   (context, v0x, v0y);
                CGContextAddLineToPoint(context, v1x, v1y);
                CGContextAddLineToPoint(context, v2x, v2y);
                CGContextAddLineToPoint(context, v0x, v0y);
                CGContextFillPath(context);
                triAngleDx += 0.1; if (triAngleDx > 2*M_PI/3) {triAngleDx = 0;}
                break;
                }
        }
    }
    return tgt;
}

- (void) bonusTargetOn {
    showBonusTarget = true;
    bonusTargetType = arc4random() % 3;
    //bonusTargetType = kBonusTriangle; //test***
}

- (void) drawFireball: (CGContextRef) context {
    
    CGRect fireballRect;
    if (shotFired) {
        switch (fireMode) {
            case kFireModeCannon:
                fireballRect = [self getFireballPosition];
                CGContextSetFillColorWithColor(context, [UIColor redColor].CGColor);
                CGContextFillEllipseInRect(context, fireballRect);
                break;
            case kFireModeMachineGun:
                fireballRect = [self getFireballPosition];
                CGContextSetFillColorWithColor(context, [UIColor whiteColor].CGColor);
                for (int b = 0; b < mgBulletCtr; b++) {
                    CGContextFillEllipseInRect(context, mgBullets[b]);
                }
                break;
            case kFireModeMortar: {
                fireballRect = [self getFireballPosition];
                CGContextSetFillColorWithColor(context, [UIColor grayColor].CGColor);
                CGContextFillEllipseInRect(context, fireballRect);
                [self drawSparks:context fireball:fireballRect];
                break;}
            case kFireModeLazer:                                        //line
                if (lazerOn) {
                    CGContextSetStrokeColorWithColor(context, [self setLazerColor]);
                    CGContextBeginPath(context);
                    CGContextMoveToPoint(context, aimFocalPt.x-2, aimFocalPt.y);
                    CGContextAddLineToPoint(context, shotEndPt.x-2, shotEndPt.y);
                    CGContextAddLineToPoint(context, shotEndPt.x+2, shotEndPt.y);
                    CGContextAddLineToPoint(context, aimFocalPt.x+2, aimFocalPt.y);
                    CGContextClosePath(context);
                    CGContextStrokePath(context);

                    CGContextSetStrokeColorWithColor(context, [self setLazerColor]);
                    CGContextMoveToPoint(context, aimFocalPt.x, aimFocalPt.y);
                    CGContextAddLineToPoint(context, shotEndPt.x, shotEndPt.y);
                    CGContextStrokePath(context);
                }
                break;
            default:
                NSLog(@"bad fireMode mojo");
                break;
        }            
            
        if ((fireBall.origin.y < -kFireballSize ||
            fireBall.origin.x < -kFireballSize ||
            fireBall.origin.x > kFieldEdge + kFireballSize) ||
            (fireMode == kFireModeLazer && !lazerOn))
        {
            shotFired = false; //offscreen
            if (fireMode == kFireModeMortar) {
                [fmodAudio stopSparks];
            }
        } else {
            [self checkCollisions: context];
        }
    }
}

- (CGRect) getFireballPosition {
        
    CGRect fireballRect;
    int fireModeSpeed;
    switch (fireMode) {
        case kFireModeCannon:       fireModeSpeed =  0; break;
        case kFireModeMachineGun:   fireModeSpeed =  5; break;
        case kFireModeMortar:       fireModeSpeed = -5; break;
        default:                    fireModeSpeed =  0; break;
    }
    fireBall.origin.y -= (kShotSpeed + fireModeSpeed);
    mSlope = tanf(shotFiredAtAngle);
    fireBall.origin.x = aimFocalPt.x + ((aimFocalPt.y - fireBall.origin.y) / mSlope);
    fireballRect = CGRectMake(fireBall.origin.x - (kFireballSize/2), fireBall.origin.y - (kFireballSize/2), kFireballSize, kFireballSize);
    
    int b = mgBulletCtr;
    if (fireMode == kFireModeMachineGun) {
        while (b > 0) {
            mgBullets[b] = mgBullets[b-1];
            b--;
        }
        mgBullets[0] = CGRectMake(fireballRect.origin.x, fireballRect.origin.y, 5, 5);
        if (mgBulletCtr < 5) {
            mgBulletCtr++;
        }
    }
    return fireballRect;
}

- (CGColor*) setLazerColor {

    CGColor *whichColor = nil;
    switch (lazerClrCtr) {
        case 0: whichColor = [UIColor whiteColor].CGColor;break;
        case 1: whichColor = [UIColor redColor].CGColor;break;
        case 2: whichColor = [UIColor orangeColor].CGColor;break;
        case 3: whichColor = [UIColor yellowColor].CGColor;break;
        case 4: whichColor = [UIColor greenColor].CGColor;break;
        case 5: whichColor = [UIColor blueColor].CGColor;break;
        case 6: whichColor = [UIColor purpleColor].CGColor;break;
        default: NSLog(@"setLazerColor bad mojo"); break;
    }
    lazerClrCtr++;
    if (lazerClrCtr > 6) {lazerClrCtr = 0;}
    return whichColor;
}

- (void) drawSparks:(CGContextRef)context fireball:(CGRect)fireballRect {
    
    CGContextSetFillColorWithColor(context, [UIColor yellowColor].CGColor);
    CGPoint sparkPt[22];
    CGPoint zeroPt = CGPointMake(0, 0);
    int i, numSparks;
    
    //draw sparks
    CGContextBeginPath(context);
    CGContextMoveToPoint(context, fireballRect.origin.x - 5, fireballRect.origin.y);
    CGContextAddLineToPoint(context, aimFocalPt.x, aimFocalPt.y);
    CGContextAddLineToPoint(context, fireballRect.origin.x + kFireballSize + 5, fireballRect.origin.y);
    CGContextClosePath(context);
    
    CGRect sparkRect = CGContextGetPathBoundingBox(context);
    if (sparkRect.size.height > 100) {sparkRect.size.height = 100;}
    
    float dist = fabsf((M_PI_2 - shotFiredAtAngle) / M_PI_2)*10;
    numSparks = 10 + ((int)dist*2);
    
    for (i=0; i<numSparks; i++) {
        sparkPt[i].x = sparkRect.origin.x + (arc4random() % (int)sparkRect.size.width);
        sparkPt[i].y = sparkRect.origin.y + (arc4random() % (int)sparkRect.size.height);
        if (!CGContextPathContainsPoint(context, sparkPt[i], kCGPathFill)) {
            sparkPt[i] = zeroPt;
        }
    }
    
    for (i=0; i<numSparks; i++) {
        if (!CGPointEqualToPoint(sparkPt[i], zeroPt)) {
            CGContextFillRect(context, CGRectMake(sparkPt[i].x, sparkPt[i].y, 2, 2));
        }
    }
    
    //set sparks vol
    float currShotDist;
    if (shotFiredAtAngle < M_PI_2) {
        currShotDist = sqrtf(pow((fireballRect.origin.x - aimFocalPt.x),2) + pow((fireballRect.origin.y - aimFocalPt.y),2));
    } else {
        currShotDist = sqrtf(pow((aimFocalPt.x - fireballRect.origin.x),2) + pow((fireballRect.origin.y - aimFocalPt.y),2));
    } 
    [fmodAudio setSparkVol:currShotDist / shotLength];

}

- (void) drawExplosions: (CGContextRef) context {
    // draw explosions
    for (int t=0; t<kNumberOfTargets; t++) {
        Targets *tgt = [targetArray objectAtIndex:t];
        if (tgt.targetHit) {
            if (t == kBonusTarget && bonusTargetType == kBonusTriangle) {
                //use fadeCtr to set size of expanding animated circle
                float circleSize = (1.0 - tgt.fadeCtr) * 100;
                CGContextSetRGBStrokeColor(context,1.0,0.0,1.0,1.0);
                CGRect circleRect = CGRectMake(tgt.tgtImageRect.origin.x - circleSize, -tgt.tgtImageRect.origin.y - circleSize, circleSize*2, circleSize*2);
                CGContextStrokeEllipseInRect(context, circleRect);
            } else {
                CGContextSaveGState(context);
                CGContextSetAlpha(context, tgt.fadeCtr);
                CGContextTranslateCTM(context, 0, tgt.tgtImage.size.height);
                CGContextScaleCTM(context, 1.0, -1.0);
                CGContextDrawImage(context, tgt.tgtImageRect, tgt.tgtImage.CGImage);
                CGContextRestoreGState(context);
            }
            tgt.fadeCtr -= 0.05;
            if (tgt.fadeCtr < 0) {
                tgt.targetHit = false;
            }
            [targetArray replaceObjectAtIndex:t withObject:tgt];
        }
    }
}

- (void) checkCollisions: (CGContextRef) context {
    
    BOOL targetHit = false;
    CGRect hitRect;
    for (int t=0; t<kNumberOfTargets; t++) {
        Targets *tgt = [targetArray objectAtIndex:t];
        if (t == kBonusTarget && showBonusTarget) {
            switch (bonusTargetType) {
                case kBonusGlass:
                    hitRect = tgt.targetRect;
                    break;
                case kBonusBatSignal:
                    hitRect = CGRectMake(tgt.targetRect.origin.x, tgt.targetRect.origin.y, batSignal.size.width, batSignal.size.height);
                    break;
                case kBonusTriangle:
                    hitRect = CGRectMake(tgt.targetRect.origin.x - tgt.targetRect.size.width + 12,
                                         tgt.targetRect.origin.y - tgt.targetRect.size.height + 12,
                                        (tgt.targetRect.size.width*2)-24, (tgt.targetRect.size.height*2)-24);
                    break;
            }
        } else {
            hitRect = tgt.targetRect;
        }

        switch (fireMode) {
            case kFireModeLazer: {
                CGContextBeginPath(context);
                CGContextMoveToPoint(context, aimFocalPt.x-4, aimFocalPt.y);
                CGContextAddLineToPoint(context, shotEndPt.x-4, shotEndPt.y);
                CGContextAddLineToPoint(context, shotEndPt.x+4, shotEndPt.y);
                CGContextAddLineToPoint(context, aimFocalPt.x+4, aimFocalPt.y);
                CGContextClosePath(context);
                if (CGContextPathContainsPoint(context, CGPointMake(hitRect.origin.x + hitRect.size.width/2, 
                                                                    hitRect.origin.y + hitRect.size.height/2), kCGPathFill)) {
                    targetHit = true;
                    [lazerTimer invalidate];
                    lazerTimer = NULL;
                }
                break;
            }
            case kFireModeMachineGun: {
                for (int b=0; b<mgBulletCtr; b++) {
                    if (CGRectIntersectsRect(hitRect, mgBullets[b])) {
                        targetHit = true;
                    }
                }
                break;
            }
            default: {// cannon and mortar
                if (CGRectIntersectsRect(hitRect, fireBall)) {
                    targetHit = true;
                }
                break;
            }
        }
        
        if (targetHit) {
            shotFired = false;
            targetHit = false;
            tgt.targetHit = true;
            tgt.tgtImage = [self loadTargetImage:t];
            float imageX = tgt.targetRect.origin.x - (tgt.tgtImage.size.width / 4);
            float imageY = -tgt.targetRect.origin.y + (tgt.tgtImage.size.height / 4);
            tgt.tgtImageRect = CGRectMake(imageX, imageY, tgt.tgtImage.size.width, tgt.tgtImage.size.height);
            float newEntrY = arc4random() % (kFieldEdge/2);
            if (t == kBonusTarget) {
                tgt.targetRect = CGRectMake(-60, newEntrY, tgt.targetRect.size.width, tgt.targetRect.size.height);
                showBonusTarget = false;
                [NSTimer scheduledTimerWithTimeInterval:20.0 target:self selector:@selector(bonusTargetOn) userInfo:nil repeats:NO];
            } else {
                switch (arc4random()%2) { //flip a coin to determine entry from left or right side
                    case 0:
                        tgt.targetRect = CGRectMake(-tgt.targetRect.size.width, newEntrY, tgt.targetRect.size.width, tgt.targetRect.size.height);
                        tgt.nextPos = CGPointMake(kFieldEdge, 0);
                        break;
                    case 1:
                        tgt.targetRect = CGRectMake(kFieldEdge+tgt.targetRect.size.width, newEntrY, tgt.targetRect.size.width, tgt.targetRect.size.height);
                        tgt.nextPos = CGPointMake(0, 0);
                        break;
                }
                tgt.targetSlopeAmt = (arc4random() % 10)+1;
            }
            
            tgt.fadeCtr = 1.0;
            [targetArray replaceObjectAtIndex:t withObject:tgt];
            
            switch (fireMode) {
                case kFireModeMortar:
                    [fmodAudio stopSparks];
                    break;
                case kFireModeLazer:
                    [fmodAudio stopLazer];
                    break;
            }

            hitCounter++;
            if (hitCounter > 10) {
                hitCounter = 0;
                fireMode++;
                if (fireMode > 3) {
                    fireMode = 0;
                }
            }
        }
    }
}

- (UIImage*) loadTargetImage:(int) whichTarget;
{
    UIImage *pickImage = nil;
    int     whichImage;
    switch (whichTarget) {
        case kBlueTarget:
            whichImage = arc4random() % 5;
            switch (whichImage) {
                case 0:
                    pickImage = bangImage;
                    [fmodAudio playBang:0];
                    break;
                case 1:
                    pickImage = powImage;
                    [fmodAudio playBang:1];
                    break;
                case 2:
                    pickImage = bamImage;
                    [fmodAudio playBang:2];
                    break;
                case 3:
                    pickImage = whamImage;
                    [fmodAudio playBang:3];
                    break;
                case 4:
                    pickImage = blammoImage;
                    [fmodAudio playBang:4];
                    break;                    
            }
            break;
        case kGreenTarget:
            whichImage = arc4random() % 3;
            switch (whichImage) {
                case 0:
                    pickImage = blamImage;
                    [fmodAudio playBang:5];
                    break;
                case 1:
                    pickImage = krackImage;
                    [fmodAudio playBang:6];
                    break;
                case 2:
                    pickImage = boomImage;
                    [fmodAudio playBang:7];
                    break;
            }        
            break;
        case kRedTarget:
            whichImage = arc4random() % 2;
            switch (whichImage) {
                case 0:
                    pickImage = zotImage;
                    [fmodAudio playBang:8];
                    break;
                case 1:
                    pickImage = kaboomImage;
                    [fmodAudio playBang:9];
                    break;
            }        
            break;
        case kBonusTarget:
            switch (bonusTargetType) {
                case kBonusGlass:
                    pickImage = glassImage;
                    [fmodAudio playBang:10];
                    break;
                case kBonusBatSignal:
                    pickImage = batImage[batImageCtr];
                    batImageCtr++; if (batImageCtr>2) {batImageCtr=0;}
                    [fmodAudio playBang:11];
                    break;
                case kBonusTriangle:
                    pickImage = QImage;
                    [fmodAudio playBang:12];
                    break;
            }        
            break;

        default: NSLog(@"loadTargetImage bad mojo"); break;
    }
    
    //AudioServicesPlaySystemSound(kSystemSoundID_Vibrate);
    return pickImage;
}

- (void) setThumbPt:(CGPoint)thumbPt
{
    int newZone  = thumbPt.x/kFieldEdge*20;
    int prevZone = aimThumbPt.x/kFieldEdge*20;
    
    if (newZone != prevZone) {
        [fmodAudio playAiming:thumbPt.x/kFieldEdge]; // pan amount
    }
    aimThumbPt = thumbPt;
}

- (void) setFocalPt:(CGPoint)focalPt;
{
    aimFocalPt = focalPt;
    //calculate aimFocalPt angle to 0,0
    mSlope = aimFocalPt.y / aimFocalPt.x; //m = Ay - By / Ax - Bx
    aimAngleToZero = atanf(mSlope);
}

- (void) setFMODaudio:(FMODaudio *)fmod;
{
    fmodAudio = fmod;
}

- (void) fireCannon:(float)angle;
{
    if (shotFired) {
        return;
    } else {
        NSLog(@"shot fired");
        shotFired = true;
        fireBall.origin = aimFocalPt;
        shotFiredAtAngle = angle;
        mSlope = tanf(shotFiredAtAngle);

        //set shot length to edge
        float x, y, b;
        if (shotFiredAtAngle < aimAngleToZero) {
            //NSLog(@"x=edge, solve for y"); //y=mx+b
            b = aimFocalPt.y + (mSlope * aimFocalPt.x);
            x = kFieldEdge;
            y = b - (mSlope * x);
        } else 
        if (shotFiredAtAngle > M_PI - aimAngleToZero) {
            //NSLog(@"x=0, solve for y");
            b = aimFocalPt.y + (mSlope * aimFocalPt.x);
            x = 0;
            y = b - (mSlope * x);
        } else {
            //NSLog(@"y=0, solve for x");
            b = aimFocalPt.y + (mSlope * aimFocalPt.x);
            y = 0;
            x = b/mSlope;
        }
        shotEndPt = CGPointMake(x, y);
        shotLength = sqrtf(pow((shotEndPt.x - aimFocalPt.x),2) + pow((shotEndPt.y - aimFocalPt.y),2));

        switch (fireMode) {
            case kFireModeCannon:
                [fmodAudio playCannonFire];
                break;
            case kFireModeMachineGun:
                mgBulletCtr = 0;
                [fmodAudio playMGFire];
                mgFireTimer = [NSTimer scheduledTimerWithTimeInterval:0.100 target:self selector:@selector(mgFireLoop) userInfo:nil repeats:YES];
                break;
            case kFireModeMortar:
                [fmodAudio playMortar];
                break;
            case kFireModeLazer:
                //turn lazerOn for one second
                lazerOn = true;
                lazerTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(lazerOff) userInfo:nil repeats:NO];
                [fmodAudio playLazer];
                break;
        }
    }
}

- (void) lazerOff {
        lazerOn = false;
}

- (void) mgFireLoop {
    if (mgFireCtr < 5 && shotFired && fireMode == kFireModeMachineGun) {
        [fmodAudio playMGFire];
        mgFireCtr++;
    } else {
        mgFireCtr = 0;
        [mgFireTimer invalidate];
        mgFireTimer = NULL;
    }
}

- (void) dealloc
{
    [targetArray release];
    [super dealloc];
}

@end
